program lblread

  use ni3_module
  use netcdf

  implicit none

  integer(2)         :: line
  integer(8)         :: np
  integer, parameter :: NMAX = 10000
  character(80)      :: str_hd
  character(256)     :: file, str
  real(8)            :: dvp, tmp(NMAX)

  real(8)            :: v
  real(8)            :: v1
  real(8)            :: v2
  real(8)            :: v1p
  real(8)            :: v2p

  integer(4)         :: ieof

  character(64) :: ncfn
  character(64) :: mode
  integer(4)    :: ncid
  character(64) :: name
  character(64) :: stdname
  character(64) :: longname
  character(64) :: units

  integer(4)         :: i
  integer(4)         :: m
  integer(4)         :: n


  if ( iargc() /= 4 ) stop 'usage: dump <filename> <v1> <v2> <line>'
  call getarg(1,file)
  call getarg(2,str)
  read(str,*) v1
  call getarg(3,str)
  read(str,*) v2
  call getarg(4,str)
  read(str,*) line

  ! read header
  open(10,file=file,status='old',form='unformatted')
  open(11,file='panel.txt',form='formatted')
  open(12,file='data1.txt',form='formatted')
  if ( line == 2 ) open(13,file='data2.txt',form='formatted')

  ncfn = 'data1.nc'
  mode = 'new'
  call ni3_open( ncfn, mode, ncid )
  !
  name     = 'wn'
  stdname  = name
  longname = 'wavenumber'
  units    = 'm-1'
  call ni3_def_dim( ncid, name, NI3_DOUBLE, NI3_UNLIMITED, &
    & stdname, longname, units )
  !
  name = 'value'
  call ni3_def_var( ncid, name, NI3_DOUBLE, 1, (/'wn'/) )


  ! read data
  m = 0
  n = 0
  read(10) str_hd
  write(11,'(A80)') str_hd
  do
    read(10,end=100) v1p, v2p, dvp, np
    write(11,200) v1p, v2p, dvp, np
    if ( np > NMAX ) stop 'increase NMAX'
    call bufin(10,ieof,tmp,np)
    if ( ieof .le. 0 ) stop 'EOF detected after panel header'
    do i = 1, np
      v = v1p + dble( dvp ) * dble( i-1 )
      if ( v < v1 .or. v > v2 ) cycle
      m = m + 1
      write(12,300) m, v, tmp(i)

      name = 'wn'
      call ni3_put_varss( ncid, name, m, v*1.0d2 )
      name = 'value'
      call ni3_put_varss( ncid, name, m, tmp(i) )
      call ni3_redef( ncid )
      call ni3_enddef( ncid )
    end do
    if ( line == 2 ) then
      call bufin(10,ieof,tmp,np)
      if ( ieof .le. 0 ) stop 'EOF detected after panel header'
      do i = 1, np
        v = v1p + dble( dvp ) * dble( i-1 )
        if ( v < v1 .or. v > v2 ) cycle
        n = n + 1
        write(13,300) n, v, tmp(i)
      end do
    end if
  end do
100 close(10)
  close(11)
  close(12)
  if ( line == 2 ) close(13)

  call ni3_close( ncid )

200 format(2(F16.11,1X),ES20.13,1X,I5)
300 format(I8,1X,F16.11,1X,ES20.13)

end program lblread

! bufin

subroutine bufin(ifile,ieof,iarray,iwords)

  implicit none

  integer(4), intent(in ) :: ifile
  integer(8), intent(out) :: iarray(iwords)
  integer(4), intent(in ) :: iwords

  !
  integer(4) :: ieof
  integer(4) :: itest

  ieof = 1
  read(ifile,end=20) iarray
  itest = min(iwords,4)
  if ( iarray(itest) == -99 ) ieof = -99
  return
20 ieof = 0
end subroutine bufin
